#include "testlib.h"
#include <iostream>
#include <sstream>

int32_t n;
std::string sherlock_num, moriarty_num;

int32_t calculateSherlockHits(std::string &sherlock_num,
                              std::string &moriarty_num) {
  ensuref(sherlock_num.size() == moriarty_num.size(), "different num length");
  int32_t hits = 0;
  for (uint32_t i = 0; i < sherlock_num.size(); ++i) {
    if(sherlock_num[i] > moriarty_num[i]) {
      ++hits;
    }
  }
  return hits;
}

int32_t calculateMoriartyHits(std::string &sherlock_num,
                              std::string &moriarty_num) {
  ensuref(sherlock_num.size() == moriarty_num.size(), "different num length");
  int32_t hits = 0;
  for (uint32_t i = 0; i < sherlock_num.size(); ++i) {
    if(sherlock_num[i] < moriarty_num[i]) {
      ++hits;
    }
  }
  return hits;
}

bool checkMoriartySequenceIsPermutation(std::string moriarty_ans) {
  std::string copy_moriarty_num = moriarty_num;
  std::sort(moriarty_ans.begin(), moriarty_ans.end());
  std::sort(copy_moriarty_num.begin(), copy_moriarty_num.end());
  return (copy_moriarty_num != moriarty_ans);
}

void readInputFile() {
  n = inf.readInt();
  inf.nextLine();
  inf.readStringTo(sherlock_num, "[0-9]+");
  inf.readStringTo(moriarty_num, "[0-9]+");
}

void checkMinHits() {
  int32_t min_hit_ouf = ouf.readInt();
  ouf.nextLine();
  int32_t min_hit_ans = ans.readInt();
  ans.nextLine();

  std::string min_ans, min_ouf;
  ouf.readStringTo(min_ouf, "[0-9]+");
  ans.readStringTo(min_ans, "[0-9]+");
  if (min_ouf.size() != n) {
    quitf(_pe, "wrong moriarty sequence length");
  }
  if (min_ans.size() != n) {
    quitf(_fail, "wrong jury moriarty sequence length");
  }

  if(checkMoriartySequenceIsPermutation(min_ouf)) {
    quitf(_wa, "moriarty sequence is not permutation");
  }
  if(checkMoriartySequenceIsPermutation(min_ans)) {
    quitf(_fail, "moriarty jury sequence is not permutation");
  }

  int32_t sherlock_hits_ouf = calculateSherlockHits(sherlock_num, min_ouf);
  if (sherlock_hits_ouf != min_hit_ouf) {
    quitf(_wa,
          "count of hits differs from count hits according pemutation");
  }

  int32_t sherlock_hits_ans = calculateSherlockHits(sherlock_num, min_ans);
  if (sherlock_hits_ans != min_hit_ans) {
    quitf(_fail,
          "jury count of hits differs from count hits according permutation");
  }

  if (sherlock_hits_ouf > sherlock_hits_ans) {
    quitf(_wa, "jury answer has less sherlock hits");
  }
  if (sherlock_hits_ouf < sherlock_hits_ans) {
    quitf(_fail, "jury answer has more sherlock hits");
  }
}

void checkMaxHits() {
  int32_t max_hit_ouf = ouf.readInt();
  ouf.nextLine();
  int32_t max_hit_ans = ans.readInt();
  ans.nextLine();

  std::string max_ans, max_ouf;
  ouf.readStringTo(max_ouf, "[0-9]+");
  ans.readStringTo(max_ans, "[0-9]+");
  if (max_ouf.size() != n) {
    quitf(_pe, "wrong moriarty sequence length");
  }
  if (max_ans.size() != n) {
    quitf(_fail, "wrong jury moriarty sequence length");
  }

  if(checkMoriartySequenceIsPermutation(max_ouf)) {
    quitf(_wa, "moriarty sequence is not permutation");
  }
  if(checkMoriartySequenceIsPermutation(max_ans)) {
    quitf(_fail, "moriarty jury sequence is not permutation");
  }

  int32_t moriarty_hits_ouf = calculateMoriartyHits(sherlock_num, max_ouf);
  if (moriarty_hits_ouf != max_hit_ouf) {
    quitf(_wa,
          "count of hits differs from count hits according pemutation");
  }

  int32_t moriarty_hits_ans = calculateMoriartyHits(sherlock_num, max_ans);
  if (moriarty_hits_ans != max_hit_ans) {
    quitf(_fail,
          "jury count of hits differs from count hits according permutation");
  }

  if (moriarty_hits_ouf > moriarty_hits_ans) {
    quitf(_fail, "jury answer has less moriarty hits");
  }
  if (moriarty_hits_ouf < moriarty_hits_ans) {
    quitf(_wa, "jury answer has more moriarty hits");
  }
}
int main(int argc, char * argv[])
{
  setName("credit cards problem checker");

  registerTestlibCmd(argc, argv);

  readInputFile();
  checkMinHits();
  checkMaxHits();
  quitf(_ok, "");
}
