#include "testlib.h"
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <algorithm>
#include <cstring>
#include <string>
#include <cmath>
#include <cassert>
#include <ctime>
#include <vector>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include "unistd.h"

using namespace std;

#define pb push_back
#define mp make_pair
#define fs first
#define sc second

struct DSU {
	int n;
	vector <int> parent;

	DSU(int size) {
		n = size;
		parent.resize(n);
		for (int i = 0; i < n; i++)
			parent[i] = i;
	}

	int getParent(int v) {
		if (parent[v] != v) {
			parent[v] = getParent(parent[v]);
		}

		return parent[v];
	}

	bool join(int u, int v) {
		u = getParent(u);
		v = getParent(v);
		if (u != v) {
			parent[u] = v;
			return true;
		}

		return false;
	}
};

map <string, string> config;

void addEdge(int u, int v, vector <vector <int> >& graph) {
	graph[u].pb(v);
	graph[v].pb(u);
}

vector <vector <int> > genPrim() {
	int n = stoi(config["n"]);
	vector <vector <int> > graph(n);
	for (int i = 1; i < n; i++)
		addEdge(rnd.next(0, i - 1), i, graph); 

	return graph;
}

vector <vector <int> > genKruskal() {
	int n = stoi(config["n"]);
	vector <vector <int> > graph(n);
	DSU dsu(n);
	for (int i = 0; i < n - 1; i++) {
		while (true) {
			int u = rnd.next(0, n - 1);
			int v = rnd.next(0, n - 1);
			if (dsu.join(u, v))  {
			    addEdge(u, v, graph);
				break;
			}
		}
	}

	return graph;
}

vector <vector <int> > genBamboo() {
	int n = stoi(config["n"]);
	vector <vector <int> > graph(n);
	for (int i = 0; i < n - 1; i++)
		addEdge(i, i + 1, graph);

	return graph;
}

vector <vector <int> > genBambooNoise() {
    if (config["length"] == "") {
    	cerr << "To use -type bamboo_noise please provide an additional parameter -length, that should be equal to the length of the main bamboo" << endl;

    	exit(1);
    }

	int n = stoi(config["n"]);
	int m = stoi(config["length"]);
	if (m > n) {
		cerr << "Number of vertices on bamboo can't exceed the total number of vertices" << endl;

		exit(1);
	}
	vector <vector <int> > graph(n);
	for (int i = 0; i < m - 1; i++)
		addEdge(i, i + 1, graph);
	for (int i = m; i < n; i++)
		addEdge(rnd.next(0, i - 1), i, graph);

	return graph;
}

vector <vector <int> > genStar() {
	int n = stoi(config["n"]);
	vector <vector <int> > graph(n);
	for (int i = 1; i < n; i++)
		addEdge(0, i, graph);

	return graph;
}

vector <vector <int> > genMulti() {
	if (config["block"] == "") {
		cerr << "To use multitest rand-prim generator provide an additional parameter -block that should be equal to the size of one small tree" << endl;

		exit(1);
	}

	int n = stoi(config["n"]);
	int block = stoi(config["block"]);
	vector <vector <int> > graph(n);
	int cur_root = 0;
	for (int i = 1; i < n; i++) {
		if ((i - 1) % block == 0) {
			addEdge(0, i, graph);
			cur_root = i;
		} else {
			addEdge(rnd.next(cur_root, i - 1), i, graph);
		}
	}

	return graph;
}

vector <vector <int> > genBrush() {
	int n = stoi(config["n"]);
	vector <vector <int> > graph(n);
	for (int i = 0; i < (n - 1) / 2; i++) {
		addEdge(i, i + 1, graph);
	}
	for (int i = (n - 1) / 2 + 1; i < n; i++) {
		addEdge(i - (n - 1) / 2 - 1, i, graph);
	}

	return graph;
}

vector <vector <int> > genBroom() {
	if (config["length"] == "") {
		cerr << "To use broom generator provide an additional parameter -length that should be equal to the size of the length of the base" << endl;

		exit(1);
	}

	int n = stoi(config["n"]);
	int m = stoi(config["length"]);
	if (m < 1 || m > n) {
		cerr << "Incorrect length of the bamboo, pick value between 1 and the total number of vertices" << endl;
		
		exit(1);
	}
	vector <vector <int> > graph(n);
	for (int i = 0; i < m - 1; i++)
		addEdge(i, i + 1, graph);
	for (int i = m; i < n; i++)
		addEdge(m - 1, i, graph);

	return graph;
}

vector <vector <int> > genBinary() {
	int n = stoi(config["n"]);
	vector <vector <int> > graph(n);
	for (int i = 1; i < n; i++)
		addEdge((i + 1) / 2 - 1, i, graph);

	return graph;
}

vector <vector <int> > gen() {
	if (config["type"] == "prim") {
		return genPrim();
	}
	if (config["type"] == "kruskal") {
		return genKruskal();
	}	
	if (config["type"] == "bamboo") {
		return genBamboo();
	}
	if (config["type"] == "bamboo_noise") {
		return genBambooNoise();
	}
	if (config["type"] == "star") {
		return genStar();
	}
	if (config["type"] == "small_multi") {
		return genMulti();
	}
    if (config["type"] == "brush") {
    	return genBrush();
    }
    if (config["type"] == "broom") {
    	return genBroom();
    }
    if (config["type"] == "binary") {
    	return genBinary();
    }

    cerr << "Unknown tree type, use -type prim|kruskal|bamboo|bamboo_noise|star|small_multi|brush|broom|binary" << endl;
    exit(1);
}

vector <pair <int, int> > getEdgesList(const vector <vector <int> >& graph) {
	vector <pair <int, int> > edges;
	for (int i = 0; i < (int) graph.size(); i++) {
		for (auto& e : graph[i]) {
			if (e > i)
				edges.pb(mp(e, i));
		}
	}

	return edges;
}

vector <pair <int, int> > renumerateVertices(int n, const vector <pair <int, int> >& edges) {
	vector <int> p(n);
	for (int i = 0; i < n; i++)
		p[i] = i;
	shuffle(p.begin(), p.end());

	vector <pair <int, int> > result;
	for (auto& e : edges) {
		result.pb(mp(p[e.fs], p[e.sc]));
	}

	return result;
}

vector <pair <int, int> > shuffleAndSwap(const vector <pair <int, int> >& edges) {
	vector <pair <int, int> > result = edges;
	shuffle(result.begin(), result.end());
	for (auto& e : result) {
		if (rnd.next(0, 1))
			swap(e.fs, e.sc);
	}

	return result;
}

void printEdgesList(const vector <pair <int, int> >& edges, int index) {
	for (auto& e : edges) {
		printf("%d %d\n", index + e.fs, index + e.sc);
	}
}

bool checkRange(int lb, int val, int rb) {
	return lb <= val && val <= rb;
}

bool validateEdgesList(int n, vector <pair <int, int> > edges) {
	if (n != (int) edges.size() + 1)
		return false;

	DSU dsu(n);
	for (int i = 0; i < n - 1; i++) {
		if (!checkRange(0, edges[i].fs, n - 1) || !checkRange(0, edges[i].sc, n - 1) || !dsu.join(edges[i].fs, edges[i].sc))
			return false;
	}

	return true;
}

void parseConfig(int argc, char ** argv) {
	for (int i = 1; i < argc - 1; i++)
		if (argv[i][0] == '-') {
			config[string(argv[i] + 1)] = string(argv[i + 1]);
		}
}

void dfs(int n, vector <vector <int> >& graph, int v, int f, vector <int>& tin, vector <int>& tout, int& cnt) {
    cnt++;
    tin[v] = cnt;
    for (auto& u : graph[v]) {
        if (u != f) {
            dfs(n, graph, u, v, tin, tout, cnt);
        }
    }

    cnt++;
    tout[v] = cnt;
}

int main (int argc, char ** argv) {
	registerGen(argc, argv, 1);
	parseConfig(argc, argv);

	if (config["n"] == "" || config["type"] == "" || config["seed"] == "") {
		cerr << "You should provide at least 3 values: -type <type of the tree>, -n <number of vertices>, -seed <seed>" << endl;

		exit(1);
	}	

	int n = stoi(config["n"]);

    if (config["maxval"] == "") {
        cerr << "You should provide max height value -maxval <maxval>" << endl;

        exit(1);
    }

    int maxval = stoi(config["maxval"]);

	vector <vector <int> > graph = gen();
	vector <pair <int, int> > edges = getEdgesList(graph);
	edges = renumerateVertices(n, edges);
	edges = shuffleAndSwap(edges);

	if (!validateEdgesList(n, edges)) {
		cerr << "Generated graph is not a tree or somethings else is incorrect. Check the code." << endl;

		exit(1);
	}
		
    vector <int> tin(n);
    vector <int> tout(n);
    int cnt = 0;
    
    dfs(n, graph, 0, -1, tin, tout, cnt);

    vector <pair <int, int> > segs;
    for (int i = 0; i < n; i++) {
        segs.pb(mp(tin[i], tout[i]));
    }
    shuffle(segs.begin(), segs.end());



    printf("%d\n", n);
    for (int i = 0; i < n; i++) {
        printf("%d %d %d\n", segs[i].fs, segs[i].sc, rnd.next(1, maxval));
    }
	  
    return 0;
}
